import {Context, DefaultState, DefaultContext, Middleware, Next} from 'koa';
import Router from 'koa-router';
import {createHash} from 'crypto';
import {promises} from 'fs'
import {extname} from 'path';
import { DocumentBody } from '../model/mod';
import path from 'path';
/**
 * content file or directory referrer
 */
export interface ContentFile{
    getHash():Promise<string>;
    createDocumentBody():Promise<DocumentBody>;
    readonly path: string;
    readonly type: string;
}
export type ContentConstructOption = {
    hash: string
}
type ContentFileConstructor =  (new (path:string,option?:ContentConstructOption) => ContentFile)&{content_type:string};
export const createDefaultClass = (type:string):ContentFileConstructor=>{
    let cons = class implements ContentFile{
        readonly path: string;
        //type = type;
        static content_type = type;
        protected hash: string| undefined;

        constructor(path:string,option?:ContentConstructOption){
            this.path = path;
            this.hash = option?.hash;
        }
        async createDocumentBody(): Promise<DocumentBody> {
            const {base,dir, name} = path.parse(this.path);
            const ret = {
                title : name,
                basepath : dir,
                additional: {},
                content_type: cons.content_type,
                filename: base,
                tags: [],
                content_hash: this.hash ?? await this.getHash(),
            } as DocumentBody;
            return ret;
        }
        get type():string{
            return cons.content_type;
        }
        async getHash():Promise<string>{
            if(this.hash !== undefined) return this.hash;
            const stat = await promises.stat(this.path);
            const hash = createHash("sha512");
            hash.update(extname(this.path));
            hash.update(stat.mode.toString());
            //if(this.desc !== undefined)
            //    hash.update(JSON.stringify(this.desc));
            hash.update(stat.size.toString());
            this.hash = hash.digest("base64");
            return this.hash;
        }
    };
    return cons;
}
let ContstructorTable:{[k:string]:ContentFileConstructor} = {};
export function registerContentReferrer(s: ContentFileConstructor){
    console.log(`registered content type: ${s.content_type}`)
    ContstructorTable[s.content_type] = s;
}
export function createContentFile(type:string,path:string,option?:ContentConstructOption){
    const constructorMethod = ContstructorTable[type];
    if(constructorMethod === undefined){
        console.log(`${type} are not in ${JSON.stringify(ContstructorTable)}`);
        throw new Error("construction method of the content type is undefined");
    }
    return new constructorMethod(path,option);
}
export function getContentFileConstructor(type:string): ContentFileConstructor|undefined{
    const ret = ContstructorTable[type];
    return ret;
}