import {Tag, TagAccessor} from '../model/tag';
import Knex from 'knex';

type DBTags = {
    name: string,
    description?: string
}

class KnexTagAccessor implements TagAccessor{
    knex:Knex<DBTags>
    constructor(knex:Knex){
        this.knex = knex;
    }
    async getTagAllList(onlyname?:boolean){
        onlyname = onlyname ?? false;
        const t:DBTags[] = await this.knex.select(onlyname ? "*" : "name").from("tags")
        return t;
    }
    async getTagByName(name:string){
        const t:DBTags[] = await this.knex.select('*').from("tags").where({name: name});
        if(t.length === 0) return undefined;
        return t[0];
    }
    async addTag(tag: Tag){
        if(await this.getTagByName(tag.name) === undefined){
            await this.knex.insert<DBTags>({
                name:tag.name,
                description:tag.description === undefined ? "" : tag.description
            }).into("tags");
            return true;
        }
        return false;
    }
    async delTag(name:string){
        if(await this.getTagByName(name) !== undefined){
            await this.knex.delete().where({name:name}).from("tags");
            return true;
        }
        return false;
    }
    async updateTag(name:string,desc:string){
        if(await this.getTagByName(name) !== undefined){
            await this.knex.update({description:desc}).where({name:name}).from("tags");
            return true;
        }
        return false;
    }
};
export const createKnexTagController = (knex:Knex):TagAccessor=>{
    return new KnexTagAccessor(knex);
}